﻿class KeywordHighlighter {
    constructor() {
        this.isActive = false; // 分析状态
        this.instance = null; // Mark.js 实例
        this.init();

        // 添加新的配置选项
        this.config = {
            highlightColors: {
                english: 'yellow',
                chinese: 'cyan',
                numbers: '#FFB6C1', // 新增数字高亮颜色
                urls: '#98FB98'     // 新增URL高亮颜色
            },
            scanInterval: 2000,     // 自动扫描间隔
            autoScan: false,        // 自动扫描开关
            excludedElements: ['script', 'style', 'pre', 'code'] // 排除的元素
        };

        // 添加新的正则表达式
        this.patterns = {
            english: /\b(?:[a-zA-Z]:)?(?:[\\/][\w.-]+)+\.\w+\b/g,
            chinese: /[\u4e00-\u9fa5]+/g,
            numbers: /\b\d+(?:\.\d+)?\b/g,
            urls: /https?:\/\/[^\s<>"]+|www\.[^\s<>"]+/g
        };

        // 初始化统计数据
        this.stats = {
            englishCount: 0,
            chineseCount: 0,
            numberCount: 0,
            urlCount: 0
        };
    }

    init() {
        // 确保 mark.js 和 DOMPurify 已加载
        if (typeof Mark === 'undefined') {
            console.error('Mark.js 未加载。请在此脚本之前引入 mark.js。');
            return;
        }

        if (typeof DOMPurify === 'undefined') {
            console.error('DOMPurify 未加载。请在此脚本之前引入 DOMPurify。');
            return;
        }

        // 添加自定义样式
        this.addCustomStyles();

        // 如果 "Analyze Keywords" 按钮不存在，则创建一个
        if (!document.getElementById('analyzeKeywordsBtn')) {
            const analyzeBtn = document.createElement('button');
            analyzeBtn.id = 'analyzeKeywordsBtn';
            analyzeBtn.innerHTML = `
                <div class="mini-cmd">
                    <div class="cmd-square">
                        <div class="cmd-slash">/</div>
                    </div>
                </div>
            `;
            analyzeBtn.style.position = 'fixed';
            analyzeBtn.style.bottom = '160px';
            analyzeBtn.style.right = '20px';
            analyzeBtn.style.zIndex = '1000';
            analyzeBtn.style.cursor = 'pointer';
            analyzeBtn.style.border = 'none';
            analyzeBtn.style.padding = '0';
            analyzeBtn.style.background = 'none';
            document.body.appendChild(analyzeBtn);

            // 添加点击事件监听器
            analyzeBtn.addEventListener('click', (e) => {
                // Prevent event from bubbling to draggable
                e.stopPropagation();
                this.toggleAnalysis(analyzeBtn);
            });

            // 添加拖拽功能
            this.makeDraggable(analyzeBtn);

            // 关闭按钮事件
            const closeBtn = analyzeBtn.querySelector('.cmd-btn-close');
            closeBtn.addEventListener('click', (e) => {
                e.stopPropagation();
                this.toggleAnalysis(analyzeBtn);
            });
        }
    }

    /**
     * 添加自定义样式
     */
    addCustomStyles() {
        const style = document.createElement('style');
        style.innerHTML = `
            .mini-cmd {
                width: 34.5px;
                height: 34.5px;
                position: relative;
                cursor: move;
                transform: translateY(-30px) translateX(10px);
            }

            .cmd-square {
                width: 100%;
                height: 100%;
                background-color: rgba(30, 30, 30, 0.5);
                border-radius: 5px;
                display: flex;
                justify-content: center;
                align-items: center;
                box-shadow: 0 0 5px rgba(0, 255, 0, 0.3);
            }

            .cmd-slash {
                color: #00ff00;
                font-family: 'Courier New', monospace;
                font-size: 20px;
                font-weight: bold;
                transform-origin: center;
                transition: all 0.3s ease;
            }

            .cmd-slash.active {
                animation: rotate-slash-fast 0.5s linear infinite,
                           blink-slash 0.5s steps(2, start) infinite;
            }

            .cmd-slash.inactive {
                animation: rotate-slash-slow 2s linear infinite,
                           blink-slash 2s steps(2, start) infinite;
            }

            @keyframes rotate-slash-fast {
                0% { transform: rotate(0deg); }
                100% { transform: rotate(360deg); }
            }

            @keyframes rotate-slash-slow {
                0% { transform: rotate(0deg); }
                100% { transform: rotate(360deg); }
            }

            @keyframes blink-slash {
                0%, 100% { opacity: 1; }
                50% { opacity: 0; }
            }

            .cmd-window {
                width: 200px;
                height: 100px;
                background-color: #1e1e1e;
                color: #00ff00;
                font-family: 'Courier New', Courier, monospace;
                border: 2px solid #333;
                border-radius: 5px;
                box-shadow: 0 0 10px rgba(0, 255, 0, 0.5);
                position: relative;
                overflow: hidden;
                animation: blink-cmd 1s steps(5, start) infinite;
            }

            .cmd-header {
                background-color: #333;
                padding: 5px;
                display: flex;
                justify-content: space-between;
                align-items: center;
                cursor: move;
            }

            .cmd-title {
                font-weight: bold;
            }

            .cmd-buttons .cmd-btn {
                width: 15px;
                height: 15px;
                display: inline-block;
                text-align: center;
                line-height: 15px;
                border: 1px solid #00ff00;
                border-radius: 50%;
                margin-left: 5px;
                transition: background-color 0.3s;
            }

            .cmd-buttons .cmd-btn:hover {
                background-color: #00ff00;
                color: #1e1e1e;
            }

            .cmd-content {
                padding: 10px;
                position: relative;
                height: calc(100% - 30px);
                display: flex;
                flex-direction: column;
                justify-content: center;
                align-items: center;
            }

            .cmd-text {
                font-size: 14px;
                margin-bottom: 10px;
                animation: typing 2s steps(20, end) infinite;
            }

            .scanner {
                width: 100%;
                height: 2px;
                background-color: #00ff00;
                position: absolute;
                top: 50%;
                left: -10%;
                animation: scan 2s linear infinite;
            }

            @keyframes blink-cmd {
                0%, 100% { border-color: #00ff00; }
                50% { border-color: #ff0000; }
            }

            @keyframes typing {
                0% { width: 0; }
                100% { width: 100%; }
            }

            @keyframes scan {
                0% { left: -100%; }
                100% { left: 100%; }
            }

            .highlight-english {
                background-color: yellow;
                color: black;
            }

            .highlight-chinese {
                background-color: cyan;
                color: black;
            }

            .highlight-english, .highlight-chinese {
                position: relative;
                z-index: 9999 !important;
                pointer-events: all !important;
            }

            .highlight-english *, .highlight-chinese * {
                pointer-events: none;
            }

            .highlight-english span, .highlight-chinese span {
                pointer-events: all !important;
                position: relative;
                z-index: 10000;
            }

            .keyword-action-menu {
                position: fixed;
                z-index: 100000 !important;
            }
        `;
        document.head.appendChild(style);
    }

    /**
     * 使按钮可拖拽
     * @param {HTMLElement} element - 需要拖拽的元素
     */
    makeDraggable(element) {
        let isDragging = false;
        let startX, startY;
        let initialLeft, initialBottom;

        const header = element.querySelector('.cmd-header');
        if (header) {
            header.style.userSelect = 'none';
            header.style.cursor = 'move';
        }

        // 按下鼠标时开始拖拽
        header.addEventListener('mousedown', (e) => {
            isDragging = true;
            startX = e.clientX;
            startY = e.clientY;

            // 获取当前位置
            const rect = element.getBoundingClientRect();
            initialLeft = rect.left;
            initialBottom = window.innerHeight - rect.bottom;
            document.body.style.userSelect = 'none'; // 防止拖动时选中文本
        });

        // 鼠标移动时更新位置
        document.addEventListener('mousemove', (e) => {
            if (isDragging) {
                const deltaX = e.clientX - startX;
                const deltaY = e.clientY - startY;

                let newLeft = initialLeft + deltaX;
                let newBottom = initialBottom - deltaY;

                // 限制按钮在视口内
                newLeft = Math.max(0, Math.min(newLeft, window.innerWidth - element.offsetWidth));
                newBottom = Math.max(0, Math.min(newBottom, window.innerHeight - element.offsetHeight));

                element.style.left = `${newLeft}px`;
                element.style.bottom = `${newBottom}px`;
            }
        });

        // 松开鼠标时停止拖拽
        document.addEventListener('mouseup', () => {
            if (isDragging) {
                isDragging = false;
                document.body.style.userSelect = 'auto';
            }
        });
    }

    toggleAnalysis(button) {
        const slashElement = button.querySelector('.cmd-slash');
        
        if (this.isActive) {
            // 立即移除活跃状态的动画
            slashElement.classList.remove('active');
            slashElement.classList.add('inactive');
            
            // 使用 requestAnimationFrame 确保动画帧同步
            requestAnimationFrame(() => {
                this.deactivateAnalysis();
            });
        } else {
            // 立即添加活跃状态的动画
            slashElement.classList.remove('inactive');
            slashElement.classList.add('active');
            
            requestAnimationFrame(() => {
                this.activateAnalysis();
            });
        }
        
        this.isActive = !this.isActive;
    }

    deactivateAnalysis() {
        if (this.instance) {
            // 立即清除所有标记
            requestAnimationFrame(() => {
                this.instance.unmark();
                this.instance = null;
            });
        }
    }

    activateAnalysis() {
        const context = document.body;
        
        // 使用 WeakMap 缓存已处理的元素
        if (!this.processedElements) {
            this.processedElements = new WeakMap();
        }

        // 初始化 Mark.js 实例
        this.instance = new Mark(context);

        // 使用 requestAnimationFrame 进行分批处理
        requestAnimationFrame(() => {
            this.instance.unmark({
                done: () => {
                    this.performHighlighting();
                }
            });
        });
    }

    performHighlighting() {
        const englishRegex = /\b(?:[a-zA-Z]:)?(?:[\\/][\w.-]+)+\.\w+\b/g;
        const chineseRegex = /[\u4e00-\u9fa5]+/g;

        // 使用 Promise 和 requestAnimationFrame 优化高亮处理
        const highlight = (regex, className) => {
            return new Promise(resolve => {
                requestAnimationFrame(() => {
                    this.instance.markRegExp(regex, {
                        element: 'span',
                        className,
                        separateWordSearch: false,
                        accuracy: 'exactly',
                        diacritics: true,
                        done: resolve
                    });
                });
            });
        };

        // 并行处理高亮
        Promise.all([
            highlight(englishRegex, 'highlight-english'),
            highlight(chineseRegex, 'highlight-chinese')
        ]).then(() => {
            requestAnimationFrame(() => {
                this.attachKeywordClickEvents();
            });
        });
    }

    attachKeywordClickEvents() {
        const highlightElements = document.querySelectorAll('.highlight-english, .highlight-chinese');
        
        highlightElements.forEach(element => {
            // 移除原有的点击事件监听器
            element.removeEventListener('click', this.showActionMenu);
            
            // 直接在高亮元素上添加点击事件，无需额外的包装器
            element.addEventListener('click', (event) => {
                event.preventDefault();
                event.stopPropagation();
                this.showActionMenu(event, element.textContent);
            }, true);
        });
    }

    showActionMenu(event, keyword) {
        event.stopPropagation();

        // 移除已有的操作菜单
        const existingMenu = document.getElementById('keywordActionMenu');
        if (existingMenu) {
            existingMenu.remove();
        }

        // 创建操作菜单容器
        const menu = document.createElement('div');
        menu.id = 'keywordActionMenu';
        menu.className = 'keyword-action-menu';

        // 菜单样式
        Object.assign(menu.style, {
            position: 'absolute',
            background: '#fff',
            border: '1px solid #ccc',
            borderRadius: '5px',
            padding: '5px',
            boxShadow: '0 2px 5px rgba(0,0,0,0.3)',
            zIndex: '1001'
        });

        // 创建操作按钮
        const inSiteSearchBtn = document.createElement('button');
        inSiteSearchBtn.textContent = '💻站内搜索';
        inSiteSearchBtn.className = 'action-btn';
        this.styleActionButton(inSiteSearchBtn);

        const mobileSiteSearchBtn = document.createElement('button');
        mobileSiteSearchBtn.textContent = '📱站内搜索';
        mobileSiteSearchBtn.className = 'action-btn';
        this.styleActionButton(mobileSiteSearchBtn);

        const googleSearchBtn = document.createElement('button');
        googleSearchBtn.innerHTML = `
            <svg viewBox="0 0 24 24" width="16" height="16" style="vertical-align: middle; margin-right: 5px;">
                <path d="M22.56 12.25c0-.78-.07-1.53-.2-2.25H12v4.26h5.92c-.26 1.37-1.04 2.53-2.21 3.31v2.77h3.57c2.08-1.92 3.28-4.74 3.28-8.09z" fill="#4285F4"/>
                <path d="M12 23c2.97 0 5.46-.98 7.28-2.66l-3.57-2.77c-.98.66-2.23 1.06-3.71 1.06-2.86 0-5.29-1.93-6.16-4.53H2.18v2.84C3.99 20.53 7.7 23 12 23z" fill="#34A853"/>
                <path d="M5.84 14.09c-.22-.66-.35-1.36-.35-2.09s.13-1.43.35-2.09V7.07H2.18C1.43 8.55 1 10.22 1 12s.43 3.45 1.18 4.93l2.85-2.22.81-.62z" fill="#FBBC05"/>
                <path d="M12 5.38c1.62 0 3.06.56 4.21 1.64l3.15-3.15C17.45 2.09 14.97 1 12 1 7.7 1 3.99 3.47 2.18 7.07l3.66 2.84c.87-2.6 3.3-4.53 6.16-4.53z" fill="#EA4335"/>
            </svg>
            Google搜索
        `;
        googleSearchBtn.className = 'action-btn';
        this.styleActionButton(googleSearchBtn);

        const bingSearchBtn = document.createElement('button');
        bingSearchBtn.innerHTML = `
            <svg viewBox="0 0 24 24" width="16" height="16" style="vertical-align: middle; margin-right: 5px;">
                <path d="M5.71 3L4.43 4.88l5.56 5.56-3.12 1.85L2 8.43v5.14l4.88 2.86 11.43-6.72L5.71 3zm.57 15.14l4.3 2.86 8.56-4.29L7.71 13l-1.43 5.14z" fill="#008373"/>
            </svg>
            必应搜索
        `;
        bingSearchBtn.className = 'action-btn';
        this.styleActionButton(bingSearchBtn);

        const perplexitySearchBtn = document.createElement('button');
        perplexitySearchBtn.innerHTML = `
            <svg viewBox="0 0 24 24" width="16" height="16" style="vertical-align: middle; margin-right: 5px;">
                <path d="M12 2C6.477 2 2 6.477 2 12c0 5.524 4.477 10 10 10s10-4.476 10-10c0-5.523-4.477-10-10-10zm0 18c-4.411 0-8-3.589-8-8s3.589-8 8-8 8 3.589 8 8-3.589 8-8 8z" fill="#6B4EFF"/>
                <path d="M13 16h-2v-2h2v2zm0-4h-2c0-3.314 3-3.314 3-5 0-1.657-1.343-3-3-3S8 5.343 8 7h-2c0-2.761 2.239-5 5-5s5 2.239 5 5c0 2.761-3 3.314-3 5z" fill="#6B4EFF"/>
            </svg>
            Perplexity搜索
        `;
        perplexitySearchBtn.className = 'action-btn';
        this.styleActionButton(perplexitySearchBtn);

        // 将按钮添加到菜单
        menu.appendChild(inSiteSearchBtn);
        menu.appendChild(mobileSiteSearchBtn);
        menu.appendChild(googleSearchBtn);
        menu.appendChild(bingSearchBtn);
        menu.appendChild(perplexitySearchBtn);

        // 将菜单添到文档主体
        document.body.appendChild(menu);

        // 位置定位在点击的元素附近
        const rect = event.target.getBoundingClientRect();
        menu.style.left = `${rect.left + window.scrollX}px`;
        menu.style.top = `${rect.bottom + window.scrollY}px`;

        // 按钮点击事件
        inSiteSearchBtn.addEventListener('click', () => {
            this.performInSiteSearch(keyword);
            menu.remove();
        });

        mobileSiteSearchBtn.addEventListener('click', () => {
            this.performMobileSiteSearch(keyword);
            menu.remove();
        });

        googleSearchBtn.addEventListener('click', () => {
            this.performGoogleSearch(keyword);
            menu.remove();
        });

        bingSearchBtn.addEventListener('click', () => {
            this.performBingSearch(keyword);
            menu.remove();
        });

        perplexitySearchBtn.addEventListener('click', () => {
            this.performPerplexitySearch(keyword);
            menu.remove();
        });

        // 防止件冒泡
        menu.addEventListener('click', (e) => e.stopPropagation());

        // 点击文档其他位置关闭菜单
        document.addEventListener('click', function handler(e) {
            if (!menu.contains(e.target)) {
                menu.remove();
                document.removeEventListener('click', handler);
            }
        });
    }

    styleActionButton(button) {
        Object.assign(button.style, {
            display: 'flex',
            alignItems: 'center',
            background: 'none',
            border: 'none',
            padding: '8px 12px',
            cursor: 'pointer',
            borderRadius: '3px',
            transition: 'background-color 0.2s',
            marginBottom: '5px',
            width: '100%',
            textAlign: 'right',
            fontSize: '14px'
        });

        button.addEventListener('mouseover', () => {
            button.style.backgroundColor = 'rgba(0, 0, 0, 0.1)';
        });

        button.addEventListener('mouseout', () => {
            button.style.backgroundColor = 'transparent';
        });
    }

    performInSiteSearch(keyword) {
        const searchUrl = `http://193.134.244.102/?search=${encodeURIComponent(keyword)}`;
        window.open(searchUrl, '_blank');
    }

    performGoogleSearch(keyword) {
        const searchUrl = `https://www.google.com/search?q=${encodeURIComponent(keyword)}`;
        window.open(searchUrl, '_blank');
    }

    performBingSearch(keyword) {
        const searchUrl = `https://www.bing.com/search?q=${encodeURIComponent(keyword)}`;
        window.open(searchUrl, '_blank');
    }

    performPerplexitySearch(keyword) {
        const searchUrl = `https://www.perplexity.ai/search?q=${encodeURIComponent(keyword)}`;
        window.open(searchUrl, '_blank');
    }

    performMobileSiteSearch(keyword) {
        const searchUrl = `http://103.74.192.62:1234/2.html?offset=0&search=${encodeURIComponent(keyword)}&sort=date_modified&ascending=0`;
        window.open(searchUrl, '_blank');
    }

    // 新增：添加配置面板
    addConfigPanel() {
        const panel = document.createElement('div');
        panel.className = 'keyword-config-panel';
        panel.innerHTML = `
            <div class="config-header">
                <h3>高亮配置</h3>
                <button class="close-config">×</button>
            </div>
            <div class="config-content">
                <div class="config-item">
                    <label>
                        <input type="checkbox" ${this.config.autoScan ? 'checked' : ''} id="autoScan">
                        自动扫描
                    </label>
                </div>
                <div class="config-item">
                    <label>扫描间隔 (ms)</label>
                    <input type="number" value="${this.config.scanInterval}" id="scanInterval">
                </div>
                <div class="config-item">
                    <label>高亮颜色</label>
                    <div class="color-inputs">
                        <input type="color" value="${this.config.highlightColors.english}" id="englishColor">
                        <label>英文</label>
                        <input type="color" value="${this.config.highlightColors.chinese}" id="chineseColor">
                        <label>中文</label>
                    </div>
                </div>
            </div>
        `;

        document.body.appendChild(panel);
        this.initConfigEvents(panel);
    }

    // 新增：统计功能
    updateStats() {
        const stats = document.createElement('div');
        stats.className = 'highlight-stats';
        stats.innerHTML = `
            <div>英文: ${this.stats.englishCount}</div>
            <div>中文: ${this.stats.chineseCount}</div>
            <div>数字: ${this.stats.numberCount}</div>
            <div>URL: ${this.stats.urlCount}</div>
        `;
        
        // 更新或添加到DOM
        const existingStats = document.querySelector('.highlight-stats');
        if (existingStats) {
            existingStats.replaceWith(stats);
        } else {
            document.body.appendChild(stats);
        }
    }

    // 新增：智能识别功能
    intelligentDetection(text) {
        const detections = {
            isCode: /^(function|class|if|for|while|switch)\b/.test(text),
            isDate: /^\d{4}[-/]\d{2}[-/]\d{2}/.test(text),
            isEmail: /^[^\s@]+@[^\s@]+\.[^\s@]+$/.test(text),
            isPhoneNumber: /^\+?[\d\s-]{10,}$/.test(text)
        };

        return detections;
    }

    // 新增：复制功能增强
    enhancedCopy(text) {
        const detection = this.intelligentDetection(text);
        let formattedText = text;

        if (detection.isCode) {
            formattedText = `\`\`\`\n${text}\n\`\`\``;
        }

        navigator.clipboard.writeText(formattedText).then(() => {
            this.showNotification('已复制到剪贴板');
        });
    }

    // 新增：通知系统
    showNotification(message, type = 'info') {
        const notification = document.createElement('div');
        notification.className = `highlight-notification ${type}`;
        notification.textContent = message;
        
        document.body.appendChild(notification);
        
        setTimeout(() => {
            notification.remove();
        }, 3000);
    }

    // 新增：性能优化
    optimizePerformance() {
        // 使用 IntersectionObserver 只处理可见区域
        const observer = new IntersectionObserver((entries) => {
            entries.forEach(entry => {
                if (entry.isIntersecting) {
                    this.processElement(entry.target);
                }
            });
        });

        // 观察所有内容块
        document.querySelectorAll('p, div, span').forEach(el => {
            observer.observe(el);
        });
    }

    // 新增：导出功能
    exportHighlights() {
        const highlights = {
            english: [],
            chinese: [],
            numbers: [],
            urls: []
        };

        document.querySelectorAll('.highlight-english').forEach(el => {
            highlights.english.push(el.textContent);
        });
        // ... 其他类型的收集

        const blob = new Blob([JSON.stringify(highlights, null, 2)], {type: 'application/json'});
        const url = URL.createObjectURL(blob);
        
        const a = document.createElement('a');
        a.href = url;
        a.download = 'highlights.json';
        a.click();
        
        URL.revokeObjectURL(url);
    }
}

// 当 DOM 完全加载后初始化 KeywordHighlighter
document.addEventListener('DOMContentLoaded', () => {
    new KeywordHighlighter();
});
